// Data buffer
#define PKGLen 64
unsigned char InBuf[PKGLen];  // input report buffer
unsigned char OutBuf[PKGLen];  // output report buffer

#define BufLen 0x8000  // interal buffer length
unsigned char SSEG1[BufLen];
unsigned char *vpData;  // point to the current location in the buffer

rc rcode;
int temp;
FILE *fp;

// global constant and variable
const int ReportPerBuf = BufLen / PKGLen;
const int BitPerRPT = 8 * PKGLen;
int CellPerPage;  // variable to store IC parameters
int PagePerIC;
int PagePerBuf;  // Buf means the internal buffer
int ReportPerPage;
int ReportPerIC;
int MaxEraseTime;  // in 10mS unit.

// common routines
int vpOutBuf;  // pointer to output buffer
void InitCmdBuf() {  // initial a command report
	int i;
	for (i=0;i<PKGLen;i++)  OutBuf[i]=0;
	OutBuf[0] = 0x5A;  // the 1st lead code
	OutBuf[1] = 0xA5;  // the 2nd lead code
	vpOutBuf = 2;  // now the pointer is here
}
void ToCmdBuf(unsigned char a) {  // push a byto into command report
	if (vpOutBuf >= (PKGLen - 2)) { // the last byte must be "End Code" 00.
	  CloseU2A();
	  fclose(fp);
	  printf("\nCommand Report Full! Press any key to exit");
	  getch();
	  exit(1);
	}
	OutBuf[vpOutBuf] = a;
	vpOutBuf++;
}
void ToWait10MS() {
	ToCmdBuf(CmdWaitN100uS);
	ToCmdBuf(100);
}
rc CheckStatusRPT() {   // check status report
	rcode = (ReadReport(InBuf));
	if (rcode)  return rcode;
	if (InBuf[0] == 0xA5)  return (InBuf[1] & 0x80);  // check status report lead code and status byte
	else  return -1;
}
rc SendCmdPKG() {
	rcode = (WriteReport(OutBuf));
	if (rcode)  return rcode;
	return (CheckStatusRPT());
}
void ExitU2A() {
	  CloseU2A();
	  printf("\nError occurs at offset %d in command reprot", ((InBuf[1] & 0x3F)-1));
	  printf("\nPress any key to exit");
	  getch();
	  exit(1);
}
// general commands
void SetPara() {  // set global parameters
	ReportPerIC = CellPerPage * PagePerIC / PKGLen;
	PagePerBuf = BufLen / CellPerPage;
	ReportPerPage = (CellPerPage / PKGLen) ? (CellPerPage / PKGLen) : 1;  // at least 1 report
}
rc SelSIO() {  // select SIO mode
	InitCmdBuf();
	ToCmdBuf(CmdSelSIO);
	return (SendCmdPKG());
}
rc SelEPP() {  // select EPP mode
	InitCmdBuf();
	ToCmdBuf(CmdSelEPP);
	return (SendCmdPKG());
}
rc PowerOn() {
	InitCmdBuf();
	ToCmdBuf(CmdPowerOn);
	return (SendCmdPKG());
}
rc PowerOff() {
	InitCmdBuf();
	ToCmdBuf(CmdPowerOff);
	return (SendCmdPKG());
}

// 8051's SFR access routines
rc WriteSFR(unsigned char data, unsigned char reg) {
	InitCmdBuf();
	ToCmdBuf(CmdSFRWrite);
	ToCmdBuf(data);
	ToCmdBuf(reg);
	if (SendCmdPKG())  ExitU2A();  // with error check
	return 0;
}
rc ReadSFR(unsigned char reg) {
	InitCmdBuf();
	ToCmdBuf(CmdSFRRead);
	ToCmdBuf(reg);
	return (SendCmdPKG());
}

// general upload/download
void LoadOutBuf() {  // load output report from internal buffer
    int i;
    for(i=0; i < PKGLen; i++)
      OutBuf[i] = *vpData++;
}
int ICAdr;	// variable to store the current address of IC
rc DownIC() {
	int Bufs, len, i;
	ICAdr = 0;  // initial IC address
	Bufs = PagePerIC / PagePerBuf;
	Bufs = Bufs ? Bufs : 1;
	for (i=0; i < Bufs; i++) {  // load internal buffer from file
	  len = fread(SSEG1, 1, BufLen, fp);
	  if (len) {
	    rcode = pfDownBuf(len);  // download one buffer
	    if (rcode)  return rcode;
	  }
	}
	return 0;
}
rc UpIC() {
	int Bufs, len, i;
	ICAdr = 0;  // initial IC address
	Bufs = ReportPerIC / ReportPerBuf;
	Bufs = Bufs ? Bufs : 1;
	for(i=0; i < Bufs; i++) {
	  if (pfUpBuf())  return 1;  // read one buffer
	  len = PKGLen * ((ReportPerIC <= ReportPerBuf) ? ReportPerIC : ReportPerBuf);
	  fwrite(SSEG1, 1, len, fp);  // write to file
	  if(ferror(fp))  return 2;
	  ICAdr = ICAdr + BufLen;  // update IC address
	}
	return 0;
}

