// Use command "CmdDStbWrite" or "CmdDStbRead" to write/read the LCD module
// The module has three control pins: R/W, RS, E and 8 bi-direction data pins.
// The using of EPP signals are nWrite(P2.0) to R/W pin, nReset(P2.2) to RS and nDSTB(P2.1) to E pin,
// and PD0~PD7(P1) to data pins.
// The nDSTB has to be initialized to low because of the E pulse is positive going.

char *s = "Hollo, world";
// The WAIT(P0.7) has to be wired to GND externally or output low internally
// to pass the busy bit check in command, or it will cause a 10ms timeout error.
rc ClrBUSY() {
	InitCmdBuf();
	ToCmdBuf(CmdPBitClr);
	ToCmdBuf(P07);
	return (SendCmdPKG());
}
// The LCD module has a RS(P2.2) pin to select its access between control register and data register.
// RS=0 is for control, RS=1 is for data.
void ToRS0() {
	ToCmdBuf(CmdPBitClr);
	ToCmdBuf(P22);
}
void ToRS1() {
	ToCmdBuf(CmdPBitSet);
	ToCmdBuf(P22);
}
void ToDStbWrite(unsigned char data) {
	ToCmdBuf(CmdDStbWrite);
	ToCmdBuf(data);  // write data
}
rc InitLCD() {  // end with data register selected
	InitCmdBuf();
	ToCmdBuf(CmdSetPWidth);
	ToCmdBuf(0x04);  // set LCD enable pulse width to ~1uS
	ToCmdBuf(CmdSFRWrite);
	ToCmdBuf(0xF0);  // Set the RS(P2.2)=0, E(P2.1)=0, R/W(P2.0)=0
	ToCmdBuf(P2);
	ToDStbWrite(0x30);  // write 0x30
	ToWait10MS();
	ToDStbWrite(0x30);  // write 0x30
	ToWait10MS();
	ToDStbWrite(0x30);  // write 0x30
	ToWait10MS();
	ToDStbWrite(0x38);  // write 0x38
	ToWait10MS();
	ToDStbWrite(0x0c);  // write 0x0c
	ToWait10MS();
	ToDStbWrite(0x06);  // write 0x06
	ToWait10MS();
	ToDStbWrite(0x01);  // write 0x01
	ToWait10MS();
	ToRS1();  // select data register
	return (SendCmdPKG());
}

int main(int argc, char* argv[]) {
    HINSTANCE hDLL = LoadLibrary("F320hid.dll");
    if (hDLL == NULL)  return 1;
    OpenU2A = (OpenU2A_type)GetProcAddress(hDLL, "OpenU2A");
    CloseU2A = (CloseU2A_type)GetProcAddress(hDLL, "CloseU2A");
    ReadReport = (ReadReport_type)GetProcAddress(hDLL, "ReadReport");
    WriteReport = (WriteReport_type)GetProcAddress(hDLL, "WriteReport");

    rcode = OpenU2A();
    if (rcode)  {FreeLibrary(hDLL);  return 1;}
    // The above codes are common to all kinds of applications

    //
    if (PowerOn())  goto error;
    if (SelEPP())  goto error;
	if (ClrBUSY()) goto error;  // make WAIT not happen
	InitLCD();
    //
	InitCmdBuf();  // initial command report
	for (; *s != '\0'; s++) {
	  ToDStbWrite(*s);  // Fill-in the message
	  ToCmdBuf(CmdWaitN100uS);  // a delay 100uS
	  ToCmdBuf(0x01);
	}
	rcode = SendCmdPKG();  // send out command report and read back the status report
  	PowerOff();

    error:
    CloseU2A();
    FreeLibrary(hDLL);
    if (rcode)  {
	  printf("\nError occurs at offset %d in command reprot", ((InBuf[1] & 0x3F)-1));
	  printf("\nPress any key to exit");
      getch();
	}
    return rcode;
}

