//
rc ResetJTAG() {  // Reset JTAG to RTI(Run-Test/Idle) state
	InitCmdBuf();
	ToCmdBuf(CmdJTAGReset);
	return (SendCmdPKG());
}
rc RTI2SIR() {  // From RTI to SIR(shift IR)
	InitCmdBuf();
	ToCmdBuf(CmdTCK_TMS1);
	ToCmdBuf(CmdTCK_TMS1);
	ToCmdBuf(CmdTCK_TMS0);
	ToCmdBuf(CmdTCK_TMS0);
	return (SendCmdPKG());
}
rc RTI2SDR() {  // From RTI to SDR(shift DR)
	InitCmdBuf();
	ToCmdBuf(CmdTCK_TMS1);
	ToCmdBuf(CmdTCK_TMS0);
	ToCmdBuf(CmdTCK_TMS0);
	return (SendCmdPKG());
}
rc E1xR2DRS() {  // from Exit1-xR to DRS(DR Scan)
	InitCmdBuf();
	ToCmdBuf(CmdTCK_TMS1);
	ToCmdBuf(CmdTCK_TMS1);
	return (SendCmdPKG());
}
rc DRS2SDR() {  // move from DRS(DR Scan) to SDR(Shift DR)
	InitCmdBuf();
	ToCmdBuf(CmdTCK_TMS0);
	ToCmdBuf(CmdTCK_TMS0);
	return (SendCmdPKG());
}
// start at SIR/SDR state, set TDI according to tdi flag, flush IR/DR "len" bit,
// stay SIR/SDR or exit to Exit1-IR/DR according to tms flag
rc xRFlush(unsigned char tdi, int len, unsigned char tms) {
	InitCmdBuf();
	if (tdi)  ToCmdBuf(CmdTDI1);  // TDI=1
	else  ToCmdBuf(CmdTDI0);  // TDI=0
	ToCmdBuf(CmdTCKsL);
	ToCmdBuf((len-1) >> 8);  ToCmdBuf(len-1);  // send "len-1" TCKs
	if (tms)  ToCmdBuf(CmdTCK_TMS1);  // last TCK with TMS=1
	else  ToCmdBuf(CmdTCK_TMS0);  // last TCK with TMS=0
	return (SendCmdPKG());
}
// start at SIR/SDR state, Write/read "len" bits which can not larger than 464 bits.
// Stay at SIR/SDR or exit to the Exit1-IR/DR state, according to the tms flag
// The write data is dummy here and, the read data is stored at the status report
rc xRRead(unsigned char dummy, int len, unsigned char tms) {
	int n, i;
	if (len > ((PKGLen - 6) * 8)) {
	  printf("\nMaxium allowed number of bits in one command report is 464!");
	  return 1;
	}
	InitCmdBuf();  // initial command report
	if (tms)  ToCmdBuf(CmdBitsRWExitL);
	else  ToCmdBuf(CmdBitsRWL);
	ToCmdBuf(len >> 8);  ToCmdBuf(len);  // read/write "len" bits
	n = (len + 7) / 8;  // need so many bytes to fill
	for (i=0; i < n; i++)  ToCmdBuf(dummy);
	return (SendCmdPKG());
}
rc InitJTAG() {
	InitCmdBuf();
	ToCmdBuf(CmdSFRWrite);
	ToCmdBuf(0xF0);  // Set TCK, TDI, TMS, TRST all 0.
	ToCmdBuf(P2);
	return (SendCmdPKG());
}
int Count0s() {
	int cnt=0, i, j;
	vpData = InBuf + 2;  // initial buffer pointer
	for (i=0; i < (PKGLen - 6); i++) {
	  if (temp = *vpData)  {
		for (j=0; j < 8; j++) {
		  if (temp & 0x01) return cnt;
		  else  {temp = temp >> 1;  cnt++;}
        }
        vpData++;
      }  else  cnt += 8;
	}
    return cnt;
}

int main(int argc, char* argv[]) {
    HINSTANCE hDLL = LoadLibrary("F320hid.dll");
    if (hDLL == NULL)  return 1;
    OpenU2A = (OpenU2A_type)GetProcAddress(hDLL, "OpenU2A");
    CloseU2A = (CloseU2A_type)GetProcAddress(hDLL, "CloseU2A");
    ReadReport = (ReadReport_type)GetProcAddress(hDLL, "ReadReport");
    WriteReport = (WriteReport_type)GetProcAddress(hDLL, "WriteReport");

    rcode = OpenU2A();
    if (rcode)  {FreeLibrary(hDLL);  return 1;}
    // The above codes are common to all kinds of applications

    //
    if (PowerOn())  goto error;
    if (SelEPP())  goto error;
	InitJTAG();

	// Find number of devices in the JTAG chain
	int i, nDevices;
	ResetJTAG();  // goto RTI state
	RTI2SIR();  // from RTI to SIR(shift IR)
	rcode = xRFlush(1, 0x100, 1);  // Send 0x100 of ones into the IR registers
	if (rcode)  goto error;		   // That makes sure all devices are in BYPASS!
								   // and end with exit to Exit1-IR state
	E1xR2DRS();  // from Exit1-IR to DRS(DR Scan)
	DRS2SDR();  // from DRS to SDR(Shift DR)
	xRFlush(0, 0x100, 0);  // Send plenty of zeros into the DR to flush them, and stay at SDR state
	// now send ones and read back enough bits for checking, Exit to Exit1-DR
	rcode = xRRead(0xFF, 0x100, 1);
	if (rcode)  goto error;
	// the number of devices is equal to the number of zero bits in buffer
	// count from bit0 of the first byte until encounter an one
	nDevices = Count0s();
	printf("\nThere are %d device(s) in the JTAG chain", nDevices);

	// Read device ID
	ResetJTAG();  // goto RTI state(that loads IDCODE into IR of all the devices)
	RTI2SDR();  // from RTI to SDR(shift DR)
	for(i=0; i < nDevices; i++) {
	  xRRead(0, 32, 0);  // Read 32 bits and stay at SDR state
	  printf("\nIDCODE for device %d is %x, %x, %x, %x", i+1, InBuf[2], InBuf[3], InBuf[4], InBuf[5]);
	}  // for a XCR3128XL-VQ100, these four bytes are 0x93, 0x90, 0x88, 0x04
  	PowerOff();

    error:
    CloseU2A();
    FreeLibrary(hDLL);
    if (rcode)  {
	  printf("\nError occurs at offset %d in command reprot", ((InBuf[1] & 0x3F)-1));
	  printf("\nPress any key to exit");
      getch();
	}
    return rcode;
}

